@@ -19,6 +19,7 @@
 //! common attributes and operations for Arrow array.
 
 use crate::{bit_iterator::BitSliceIterator, bitmap::Bitmap};
+use arrow_buffer::bit_chunk_iterator::BitChunks;
 use arrow_buffer::{bit_util, ArrowNativeType, Buffer, MutableBuffer};
 use arrow_schema::{ArrowError, DataType, IntervalUnit, UnionMode};
 use half::f16;
@@ -286,7 +287,7 @@ impl ArrayData {
     /// # Safety
     ///
     /// The input values *must* form a valid Arrow array for
-    /// `data_type`, or undefined behavior can results.
+    /// `data_type`, or undefined behavior can result.
     ///
     /// Note: This is a low level API and most users of the arrow
     /// crate should create arrays using the methods in the `array`
@@ -318,19 +319,20 @@ impl ArrayData {
 
         // Provide a force_validate mode
         #[cfg(feature = "force_validate")]
-        new_self.validate_full().unwrap();
+        new_self.validate_data().unwrap();
         new_self
     }
 
-    /// Create a new ArrayData, validating that the provided buffers
-    /// form a valid Arrow array of the specified data type.
+    /// Create a new ArrayData, validating that the provided buffers form a valid
+    /// Arrow array of the specified data type.
     ///
     /// If the number of nulls in `null_bit_buffer` is 0 then the null_bit_buffer
     /// is set to `None`.
     ///
-    /// Note: This is a low level API and most users of the arrow
-    /// crate should create arrays using the methods in the `array`
-    /// module.
+    /// Internally this calls through to [`Self::validate_data`]
+    ///
+    /// Note: This is a low level API and most users of the arrow crate should create
+    /// arrays using the builders found in [arrow_array](https://docs.rs/arrow-array)
     pub fn try_new(
         data_type: DataType,
         len: usize,
@@ -366,7 +368,10 @@ impl ArrayData {
         };
 
         // As the data is not trusted, do a full validation of its contents
-        new_self.validate_full()?;
+        // We don't need to validate children as we can assume that the
+        // [`ArrayData`] in `child_data` have already been validated through
+        // a call to `ArrayData::try_new` or created using unsafe
+        new_self.validate_data()?;
         Ok(new_self)
     }
 
@@ -617,8 +622,8 @@ impl ArrayData {
     /// contents of the buffers (e.g. that all offsets for UTF8 arrays
     /// are within the bounds of the values buffer).
     ///
-    /// See [ArrayData::validate_full] to validate fully the offset content
-    /// and the validitiy of utf8 data
+    /// See [ArrayData::validate_data] to validate fully the offset content
+    /// and the validity of utf8 data
     pub fn validate(&self) -> Result<(), ArrowError> {
         // Need at least this mich space in each buffer
         let len_plus_offset = self.len + self.offset;
@@ -954,35 +959,35 @@ impl ArrayData {
         Ok(values_data)
     }
 
-    /// "expensive" validation that ensures:
+    /// Validate that the data contained within this [`ArrayData`] is valid
     ///
     /// 1. Null count is correct
     /// 2. All offsets are valid
     /// 3. All String data is valid UTF-8
     /// 4. All dictionary offsets are valid
     ///
-    /// Does not (yet) check
-    /// 1. Union type_ids are valid see [#85](https://github.com/apache/arrow-rs/issues/85)
-    /// Note calls `validate()` internally
-    pub fn validate_full(&self) -> Result<(), ArrowError> {
-        // Check all buffer sizes prior to looking at them more deeply in this function
+    /// Internally this calls:
+    ///
+    /// * [`Self::validate`]
+    /// * [`Self::validate_nulls`]
+    /// * [`Self::validate_values`]
+    ///
+    /// Note: this does not recurse into children, for a recursive variant
+    /// see [`Self::validate_full`]
+    pub fn validate_data(&self) -> Result<(), ArrowError> {
         self.validate()?;
 
-        let null_bitmap_buffer = self
-            .null_bitmap
-            .as_ref()
-            .map(|null_bitmap| null_bitmap.buffer_ref());
-
-        let actual_null_count = count_nulls(null_bitmap_buffer, self.offset, self.len);
-        if actual_null_count != self.null_count {
-            return Err(ArrowError::InvalidArgumentError(format!(
-                "null_count value ({}) doesn't match actual number of nulls in array ({})",
-                self.null_count, actual_null_count
-            )));
-        }
-
+        self.validate_nulls()?;
         self.validate_values()?;
+        Ok(())
+    }
 
+    /// Performs a full recursive validation of this [`ArrayData`] and all its children
+    ///
+    /// This is equivalent to calling [`Self::validate_data`] on this [`ArrayData`]
+    /// and all its children recursively
+    pub fn validate_full(&self) -> Result<(), ArrowError> {
+        self.validate_data()?;
         // validate all children recursively
         self.child_data
             .iter()
@@ -995,10 +1000,127 @@ impl ArrayData {
                     ))
                 })
             })?;
+        Ok(())
+    }
+
+    /// Validates the values stored within this [`ArrayData`] are valid
+    /// without recursing into child [`ArrayData`]
+    ///
+    /// Does not (yet) check
+    /// 1. Union type_ids are valid see [#85](https://github.com/apache/arrow-rs/issues/85)
+    /// Validates the the null count is correct and that any
+    /// nullability requirements of its children are correct
+    pub fn validate_nulls(&self) -> Result<(), ArrowError> {
+        let nulls = self.null_buffer();
+
+        let actual_null_count = count_nulls(nulls, self.offset, self.len);
+        if actual_null_count != self.null_count {
+            return Err(ArrowError::InvalidArgumentError(format!(
+                "null_count value ({}) doesn't match actual number of nulls in array ({})",
+                self.null_count, actual_null_count
+            )));
+        }
+
+        // In general non-nullable children should not contain nulls, however, for certain
+        // types, such as StructArray and FixedSizeList, nulls in the parent take up
+        // space in the child. As such we permit nulls in the children in the corresponding
+        // positions for such types
+        match &self.data_type {
+            DataType::List(f) | DataType::LargeList(f) | DataType::Map(f, _) => {
+                if !f.is_nullable() {
+                    self.validate_non_nullable(None, 0, &self.child_data[0])?
+                }
+            }
+            DataType::FixedSizeList(field, len) => {
+                let child = &self.child_data[0];
+                if !field.is_nullable() {
+                    match nulls {
+                        Some(nulls) => {
+                            let element_len = *len as usize;
+                            let mut buffer =
+                                MutableBuffer::new_null(element_len * self.len);
+
+                            // Expand each bit within `null_mask` into `element_len`
+                            // bits, constructing the implicit mask of the child elements
+                            for i in 0..self.len {
+                                if !bit_util::get_bit(nulls.as_ref(), self.offset + i) {
+                                    continue;
+                                }
+                                for j in 0..element_len {
+                                    bit_util::set_bit(
+                                        buffer.as_mut(),
+                                        i * element_len + j,
+                                    )
+                                }
+                            }
+                            let mask = buffer.into();
+                            self.validate_non_nullable(Some(&mask), 0, child)?;
+                        }
+                        None => self.validate_non_nullable(None, 0, child)?,
+                    }
+                }
+            }
+            DataType::Struct(fields) => {
+                for (field, child) in fields.iter().zip(&self.child_data) {
+                    if !field.is_nullable() {
+                        self.validate_non_nullable(nulls, self.offset, child)?
+                    }
+                }
+            }
+            _ => {}
+        }
 
         Ok(())
     }
 
+    /// Verifies that `child` contains no nulls not present in `mask`
+    fn validate_non_nullable(
+        &self,
+        mask: Option<&Buffer>,
+        offset: usize,
+        data: &ArrayData,
+    ) -> Result<(), ArrowError> {
+        let mask = match mask {
+            Some(mask) => mask.as_ref(),
+            None => return match data.null_count {
+                0 => Ok(()),
+                _ => Err(ArrowError::InvalidArgumentError(format!(
+                    "non-nullable child of type {} contains nulls not present in parent {}",
+                    data.data_type(),
+                    self.data_type
+                ))),
+            },
+        };
+
+        match data.null_buffer() {
+            Some(nulls) => {
+                let mask = BitChunks::new(mask, offset, data.len);
+                let nulls = BitChunks::new(nulls.as_ref(), data.offset, data.len);
+                mask
+                    .iter()
+                    .zip(nulls.iter())
+                    .chain(std::iter::once((
+                        mask.remainder_bits(),
+                        nulls.remainder_bits(),
+                    ))).try_for_each(|(m, c)| {
+                    if (m & !c) != 0 {
+                        return Err(ArrowError::InvalidArgumentError(format!(
+                            "non-nullable child of type {} contains nulls not present in parent",
+                            data.data_type()
+                        )))
+                    }
+                    Ok(())
+                })
+            }
+            None => Ok(()),
+        }
+    }
+
+    /// Validates the values stored within this [`ArrayData`] are valid
+    /// without recursing into child [`ArrayData`]
+    ///
+    /// Does not (yet) check
+    /// 1. Union type_ids are valid see [#85](https://github.com/apache/arrow-rs/issues/85)
     pub fn validate_values(&self) -> Result<(), ArrowError> {
         match &self.data_type {
             DataType::Utf8 => self.validate_utf8::<i32>(),
