@@ -91,7 +91,7 @@ fn create_array(
                     read_buffer(buffers.get(buffer_index + 1), data, compression_codec)?,
                     read_buffer(buffers.get(buffer_index + 2), data, compression_codec)?,
                 ],
-            );
+            )?;
             node_index += 1;
             buffer_index += 3;
             array
@@ -104,7 +104,7 @@ fn create_array(
                     read_buffer(buffers.get(buffer_index), data, compression_codec)?,
                     read_buffer(buffers.get(buffer_index + 1), data, compression_codec)?,
                 ],
-            );
+            )?;
             node_index += 1;
             buffer_index += 2;
             array
@@ -131,7 +131,7 @@ fn create_array(
             node_index = triple.1;
             buffer_index = triple.2;
 
-            create_list_array(list_node, data_type, &list_buffers, triple.0)
+            create_list_array(list_node, data_type, &list_buffers, triple.0)?
         }
         FixedSizeList(ref list_field, _) => {
             let list_node = nodes.get(node_index);
@@ -156,7 +156,7 @@ fn create_array(
             node_index = triple.1;
             buffer_index = triple.2;
 
-            create_list_array(list_node, data_type, &list_buffers, triple.0)
+            create_list_array(list_node, data_type, &list_buffers, triple.0)?
         }
         Struct(struct_fields) => {
             let struct_node = nodes.get(node_index);
@@ -220,7 +220,7 @@ fn create_array(
                 data_type,
                 &index_buffers,
                 value_array.clone(),
-            )
+            )?
         }
         Union(fields, field_type_ids, mode) => {
             let union_node = nodes.get(node_index);
@@ -305,7 +305,7 @@ fn create_array(
                     read_buffer(buffers.get(buffer_index), data, compression_codec)?,
                     read_buffer(buffers.get(buffer_index + 1), data, compression_codec)?,
                 ],
-            );
+            )?;
             node_index += 1;
             buffer_index += 2;
             array
@@ -397,7 +397,7 @@ fn create_primitive_array(
     field_node: &crate::FieldNode,
     data_type: &DataType,
     buffers: &[Buffer],
-) -> ArrayRef {
+) -> Result<ArrayRef, ArrowError> {
     let length = field_node.length() as usize;
     let null_buffer = (field_node.null_count() > 0).then_some(buffers[0].clone());
     let array_data = match data_type {
@@ -407,8 +407,7 @@ fn create_primitive_array(
                 .len(length)
                 .buffers(buffers[1..3].to_vec())
                 .null_bit_buffer(null_buffer)
-                .build()
-                .unwrap()
+                .build()?
         }
         FixedSizeBinary(_) => {
             // read 2 buffers: null buffer (optional) and data buffer
@@ -416,8 +415,7 @@ fn create_primitive_array(
                 .len(length)
                 .add_buffer(buffers[1].clone())
                 .null_bit_buffer(null_buffer)
-                .build()
-                .unwrap()
+                .build()?
         }
         Int8
         | Int16
@@ -434,19 +432,16 @@ fn create_primitive_array(
                     .len(length)
                     .add_buffer(buffers[1].clone())
                     .null_bit_buffer(null_buffer)
-                    .build()
-                    .unwrap();
+                    .build()?;
                 let values = Arc::new(Int64Array::from(data)) as ArrayRef;
-                // this cast is infallible, the unwrap is safe
-                let casted = cast(&values, data_type).unwrap();
+                let casted = cast(&values, data_type)?;
                 casted.into_data()
             } else {
                 ArrayData::builder(data_type.clone())
                     .len(length)
                     .add_buffer(buffers[1].clone())
                     .null_bit_buffer(null_buffer)
-                    .build()
-                    .unwrap()
+                    .build()?
             }
         }
         Float32 => {
@@ -456,19 +451,16 @@ fn create_primitive_array(
                     .len(length)
                     .add_buffer(buffers[1].clone())
                     .null_bit_buffer(null_buffer)
-                    .build()
-                    .unwrap();
+                    .build()?;
                 let values = Arc::new(Float64Array::from(data)) as ArrayRef;
-                // this cast is infallible, the unwrap is safe
-                let casted = cast(&values, data_type).unwrap();
+                let casted = cast(&values, data_type)?;
                 casted.into_data()
             } else {
                 ArrayData::builder(data_type.clone())
                     .len(length)
                     .add_buffer(buffers[1].clone())
                     .null_bit_buffer(null_buffer)
-                    .build()
-                    .unwrap()
+                    .build()?
             }
         }
         Boolean
@@ -483,8 +475,7 @@ fn create_primitive_array(
             .len(length)
             .add_buffer(buffers[1].clone())
             .null_bit_buffer(null_buffer)
-            .build()
-            .unwrap(),
+            .build()?,
         Interval(IntervalUnit::MonthDayNano) | Decimal128(_, _) => {
             let buffer = get_aligned_buffer::<i128>(&buffers[1], length);
 
@@ -493,8 +484,7 @@ fn create_primitive_array(
                 .len(length)
                 .add_buffer(buffer)
                 .null_bit_buffer(null_buffer)
-                .build()
-                .unwrap()
+                .build()?
         }
         Decimal256(_, _) => {
             let buffer = get_aligned_buffer::<i256>(&buffers[1], length);
@@ -504,13 +494,12 @@ fn create_primitive_array(
                 .len(length)
                 .add_buffer(buffer)
                 .null_bit_buffer(null_buffer)
-                .build()
-                .unwrap()
+                .build()?
         }
         t => unreachable!("Data type {:?} either unsupported or not primitive", t),
     };
 
-    make_array(array_data)
+    Ok(make_array(array_data))
 }
 
 /// Checks if given `Buffer` is properly aligned with `T`.
@@ -538,7 +527,7 @@ fn create_list_array(
     data_type: &DataType,
     buffers: &[Buffer],
     child_array: ArrayRef,
-) -> ArrayRef {
+) -> Result<ArrayRef, ArrowError> {
     let null_buffer = (field_node.null_count() > 0).then_some(buffers[0].clone());
     let length = field_node.length() as usize;
     let child_data = child_array.into_data();
@@ -556,7 +545,7 @@ fn create_list_array(
 
         _ => unreachable!("Cannot create list or map array from {:?}", data_type),
     };
-    make_array(builder.build().unwrap())
+    Ok(make_array(builder.build()?))
 }
 
 /// Reads the correct number of buffers based on list type and null_count, and creates a
@@ -566,7 +555,7 @@ fn create_dictionary_array(
     data_type: &DataType,
     buffers: &[Buffer],
     value_array: ArrayRef,
-) -> ArrayRef {
+) -> Result<ArrayRef, ArrowError> {
     if let Dictionary(_, _) = *data_type {
         let null_buffer = (field_node.null_count() > 0).then_some(buffers[0].clone());
         let builder = ArrayData::builder(data_type.clone())
@@ -575,7 +564,7 @@ fn create_dictionary_array(
             .add_child_data(value_array.into_data())
             .null_bit_buffer(null_buffer);
 
-        make_array(unsafe { builder.build_unchecked() })
+        Ok(make_array(builder.build()?))
     } else {
         unreachable!("Cannot create dictionary array from {:?}", data_type)
     }
