@@ -377,7 +377,7 @@ impl IpcDataGenerator {
                 offset,
                 array.len(),
                 array.null_count(),
-                &compression_codec,
+                compression_codec,
                 write_options,
             )?;
         }
@@ -452,7 +452,7 @@ impl IpcDataGenerator {
             0,
             array_data.len(),
             array_data.null_count(),
-            &compression_codec,
+            compression_codec,
             write_options,
         )?;
 
@@ -793,15 +793,13 @@ impl<W: Write> StreamWriter<W> {
     /// # fn main() -> Result<(), ArrowError> {
     /// // The result we expect from an empty schema
     /// let expected = vec![
-    ///     255, 255, 255, 255,  64,   0,   0,   0,
+    ///     255, 255, 255, 255,  48,   0,   0,   0,
     ///      16,   0,   0,   0,   0,   0,  10,   0,
-    ///      14,   0,  12,   0,  11,   0,   4,   0,
-    ///      10,   0,   0,   0,  20,   0,   0,   0,
-    ///       0,   0,   0,   1,   4,   0,  10,   0,
-    ///      12,   0,   0,   0,   8,   0,   4,   0,
-    ///      10,   0,   0,   0,   8,   0,   0,   0,
-    ///       8,   0,   0,   0,   0,   0,   0,   0,
-    ///       0,   0,   0,   0,   0,   0,   0,   0,
+    ///      12,   0,  10,   0,   9,   0,   4,   0,
+    ///      10,   0,   0,   0,  16,   0,   0,   0,
+    ///       0,   1,   4,   0,   8,   0,   8,   0,
+    ///       0,   0,   4,   0,   8,   0,   0,   0,
+    ///       4,   0,   0,   0,   0,   0,   0,   0,
     ///     255, 255, 255, 255,   0,   0,   0,   0
     /// ];
     ///
@@ -1058,7 +1056,7 @@ fn write_array_data(
     offset: i64,
     num_rows: usize,
     null_count: usize,
-    compression_codec: &Option<CompressionCodec>,
+    compression_codec: Option<CompressionCodec>,
     write_options: &IpcWriteOptions,
 ) -> Result<i64, ArrowError> {
     let mut offset = offset;
@@ -1234,7 +1232,7 @@ fn write_buffer(
     buffers: &mut Vec<crate::Buffer>, // output buffer descriptors
     arrow_data: &mut Vec<u8>,         // output stream
     offset: i64,                      // current output stream offset
-    compression_codec: &Option<CompressionCodec>,
+    compression_codec: Option<CompressionCodec>,
 ) -> Result<i64, ArrowError> {
     let len: i64 = match compression_codec {
         Some(compressor) => compressor.compress_to_vec(buffer, arrow_data)?,
@@ -1799,7 +1797,7 @@ mod tests {
                     Arc::new(strings) as ArrayRef,
                 ),
                 (
-                    Field::new("c", DataType::Int32, false),
+                    Field::new("c", DataType::Int32, true),
                     Arc::new(ints) as ArrayRef,
                 ),
             ]);
