@@ -21,7 +21,6 @@
 
 #[cfg(feature = "ffi")]
 mod ffi;
-mod ord;
 
 // --------------------- Array & ArrayData ---------------------
 pub use arrow_array::array::*;
@@ -39,4 +38,4 @@ pub use self::ffi::{export_array_into_raw, make_array_from_raw};
 
 // --------------------- Array's values comparison ---------------------
 
-pub use self::ord::{build_compare, DynComparator};
+pub use arrow_ord::ord::{build_compare, DynComparator};
