@@ -713,7 +713,7 @@ mod tests {
             Field::new(
                 "c1",
                 DataType::Struct(vec![
-                    Field::new("c11", DataType::Int32, false),
+                    Field::new("c11", DataType::Int32, true),
                     Field::new(
                         "c12",
                         DataType::Struct(vec![Field::new("c121", DataType::Utf8, false)]),
@@ -727,7 +727,7 @@ mod tests {
 
         let c1 = StructArray::from(vec![
             (
-                Field::new("c11", DataType::Int32, false),
+                Field::new("c11", DataType::Int32, true),
                 Arc::new(Int32Array::from(vec![Some(1), None, Some(5)])) as ArrayRef,
             ),
             (
